{

  - Partly based on the public domain codes by:

          - Scott Bussinger
          - Carley Phillips

    which were written for Pascal.

  - Routines in this unit utilise certain global system variables and routines
    which are specific to Delphi.
}
unit mydate;

interface

type
  DayFormat = (LongDay, ShortDay);
  
  thisTime = LongInt;
  JulianDate = LongInt;

  DDMMYYYY = string[10];  {dd/mm/yyyy as in 30/11/2001.}
const
  BadTime = $FFFFFFFF;

  SecondsInDay = 86400;      {number of seconds in a day}
  SecondsInHour = 3600;      {number of seconds in an hour}
  SecondsInMinute = 60;      {number of seconds in a minute}
  HoursInDay = 24;           {number of hours in a day}
  MinutesInHour = 60;        {number of minutes in an hour}

  MinYear = 1600;
  MaxYear = 3999;
  MinDate = $00000000;     {= 01/01/1600}
  MaxDate = $000D6025;     {= 12/31/3999}
  Date1900 = $0001AC05;    {= 01/01/1900}
  Date1980 = $00021E28;    {= 01/01/1980}
  Date2000 = $00023AB1;    {= 01/01/2000}
  BadDate = $FFFFFFFF;

  First2Months = 59;         {1600 was a leap year}

  Threshold2000 : Integer  = 1900;

function DDMMYY2DDMMMYY(var S : string) : boolean;

function IsLeapYear(Year : Integer) : Boolean;
function DaysInMonth(Month, Year : Integer) : Integer;
function ValidDate(Day, Month, Year : Integer) : Boolean;
function DMYtoDate(Day, Month, Year : Integer) : JulianDate;
procedure DateToDMY(Julian : JulianDate; var Day, Month, Year : Integer);
function IncDecFromDateStr(Dir : integer; var dStr : DDMMYYYY; var Day : string) : boolean;

function HMStoTime(Hours, Minutes, Seconds : Byte) : thisTime;
procedure TimeToHMS(T : thisTime; var Hours, Minutes, Seconds : Byte);
procedure TimeDiff(Time1, Time2 : thisTime; var Hours, Minutes, Seconds : Byte);

procedure StrToDMY(dStr : string; var D, M, Y : integer);

function DayOfWeekFromDateStr(dStr : string; DF : DayFormat) : string;

implementation
uses SysUtils,
     myString;

{
  Converts 99/99/99 to 11/XXX/99.
  ====> 11/12/97 to 11-DEC-97.

  Assuming that S represents a valid date.
}
function DDMMYY2DDMMMYY(var S : string) : boolean;
var
  Month : integer;
  i, j : byte;
begin
  DDMMYY2DDMMMYY := True;
  
  i := pos('/', S);
  S[i] := '-';

  j := pos('/', S);
  S[j] := '-';

  Month := StrToInt(copy(S, i+1, j-i-1));

  if not (Month in [1..12]) then
    DDMMYY2DDMMMYY := False;

  delete(S, i+1, j-i-1);
  insert(ShortMonthNames[Month], S, i+1);

end;

function IsLeapYear(Year : Integer) : Boolean;
begin
  IsLeapYear := (Year mod 4 = 0) and (Year mod 4000 <> 0) and
    ((Year mod 100 <> 0) or (Year mod 400 = 0));
end;

function DaysInMonth(Month, Year : Integer) : Integer;
begin
  if Word(Year) < 100 then begin
    Inc(Year, 1900);
    if Year < Threshold2000 then
      Inc(Year, 100);
  end;

  case Month of
    1, 3, 5, 7, 8, 10, 12 :
      DaysInMonth := 31;
    4, 6, 9, 11 :
      DaysInMonth := 30;
    2 :
      DaysInMonth := 28+Ord(IsLeapYear(Year));
  else
    DaysInMonth := 0;
  end;
end;

function ValidDate(Day, Month, Year : Integer) : Boolean;
  {-Verify that day, month, year is a valid date}
begin
  if Word(Year) < 100 then begin
    Inc(Year, 1900);
    if Year < Threshold2000 then
      Inc(Year, 100);
  end;

  if (Day < 1) or (Year < MinYear) or (Year > MaxYear) then
    ValidDate := False
  else case Month of
    1..12 :
      ValidDate := Day <= DaysInMonth(Month, Year);
  else
    ValidDate := False;
  end
end;

function DMYtoDate(Day, Month, Year : Integer) : JulianDate;
 {-Convert from day, month, year to a julian date}
begin
  if Word(Year) < 100 then begin
    Inc(Year, 1900);
    if Year < Threshold2000 then
      Inc(Year, 100);
  end;

  if not ValidDate(Day, Month, Year) then
    DMYtoDate := BadDate
  else if (Year = MinYear) and (Month < 3) then
    if Month = 1 then
      DMYtoDate := Pred(Day)
    else
      DMYtoDate := Day+30
  else begin
    if Month > 2 then
      Dec(Month, 3)
    else begin
      Inc(Month, 9);
      Dec(Year);
    end;
    Dec(Year, MinYear);
    DMYtoDate :=
        ((LongInt(Year div 100)*146097) div 4)+
        ((LongInt(Year mod 100)*1461) div 4)+
        (((153*Month)+2) div 5)+Day+First2Months;
  end;
end;

procedure DateToDMY(Julian : JulianDate; var Day, Month, Year : Integer);
 {-Convert from a julian date to month, day, year}
var
  I, J : LongInt;
begin
  if Julian = BadDate then begin
    Day := 0;
    Month := 0;
    Year := 0;
  end
  else if Julian <= First2Months then begin
    Year := MinYear;
    if Julian <= 30 then begin
      Month := 1;
      Day := Succ(Julian);
    end
    else begin
      Month := 2;
      Day := Julian-30;
    end;
  end
  else begin
    I := (4*LongInt(Julian-First2Months))-1;
    J := (4*((I mod 146097) div 4))+3;
    Year := (100*(I div 146097))+(J div 1461);
    I := (5*(((J mod 1461)+4) div 4))-3;

    Month := I div 153;
    Day := ((I mod 153)+5) div 5;
    if Month < 10 then
      Inc(Month, 3)
    else begin
      Dec(Month, 9);
      Inc(Year);
    end;
    Inc(Year, MinYear);
  end;
end;

{
  12/11/97.

  Dir:  the next day (Up) or the previous day (Down).
  dStr: represents the input date string in dd/mm/yy format.
        represents the output date string in dd/mm/yy format.
  Day:  represents date (Mon, Tue etc) of dStr.

  Return True if dStr represents a valid date. False otherwise.
  When dStr represents a valid date, dStr is changed to the next or
  previous day and Day's value is defined.

  First revision: 03/1/98. 
}
function IncDecFromDateStr(Dir : integer; var dStr : DDMMYYYY; var Day : string) : boolean;
var
  d, m, y : integer;
  i, j : byte;

  theDate : longint;
begin
  IncDecFromDateStr := True;
  Day := '';

  {Break the string into date/month/year.}
  i := pos('/', dStr);
  d := StrToInt(copy(dStr, 1, i - 1));
  dStr[i] := '@';
  j := pos('/', dStr);
  m := StrToInt(copy(dStr, i+1, j-i-1));
  y := StrToInt(copy(dStr, j+1, length(dStr)-j));

  if not ValidDate(d, m, y) then
    begin
      IncDecFromDateStr := False;
      exit;
    end;

  {Go to next day or previous day.}
  theDate := DMYtoDate(d, m, y) + Dir;

  DateToDMY(theDate, d, m, y);

  dStr := IntToStr(d) + '/' + IntToStr(m) + '/' + IntToStr(y);

  Day := ShortDayNames[(
                         (
                           (theDate + 6) mod 7
                         ) + 1
                       )];
end;

function HMStoTime(Hours, Minutes, Seconds : Byte) : thisTime;
var
  T : thisTime;
begin
  Hours := Hours mod HoursInDay;
  T := (LongInt(Hours)*SecondsInHour)+(LongInt(Minutes)*SecondsInMinute)+Seconds;
  HMStoTime := T mod SecondsInDay;
end;

procedure TimeToHMS(T : thisTime; var Hours, Minutes, Seconds : Byte);
begin
  if T = BadTime then begin
    Hours := 0;
    Minutes := 0;
    Seconds := 0;
  end
  else begin
    Hours := T div SecondsInHour;
    Dec(T, LongInt(Hours)*SecondsInHour);
    Minutes := T div SecondsInMinute;
    Dec(T, LongInt(Minutes)*SecondsInMinute);
    Seconds := T;
  end;
end;

procedure TimeDiff(Time1, Time2 : thisTime; var Hours, Minutes, Seconds : Byte);
var
  T : thisTime;
begin
  if Time1 > Time2 then
   begin
     T := Time1-Time2;
     TimeToHMS(T, Hours, Minutes, Seconds);
     Hours := 12 - Hours;
   end
  else
   begin
     T := Time2-Time1;
     TimeToHMS(T, Hours, Minutes, Seconds);
   end;

end;

{
  21/12/97.

  Input variable dStr is assumed to contain a valid Australian date string.
  dStr should be validated before passed to this function.

  Breaks dStr into date, month, year components.

  First revision: 4/1/98.
}
procedure StrToDMY(dStr : string; var D, M, Y : integer);
var
  i, j : byte;
begin
  i := pos('/', dStr);
  D := StrToInt( Remove_Embeded_Spaces(copy(dStr, 1, i - 1)) );


  dStr[i] := '@';
  j := pos('/', dStr);
  M := StrToInt( Remove_Embeded_Spaces(copy(dStr, i+1, j-i-1)) );
  Y := StrToInt( Remove_Embeded_Spaces(copy(dStr, j+1, length(dStr)-j)) );
end;

{
   4/1/98.

   - Returns the day name for the date string.

   - dStr: the string represents the date; e.g. 12/12/97, 12/12/1997.
   - DF: how should the day name be returned: full or abbreviated;
         for example: Mon or Monday.

   - Assumption: the dStr represents a valid date. No validity checking is
     performed on the date string.
}
function DayOfWeekFromDateStr(dStr : string; DF : DayFormat) : string;
var
  DD, MM, YY : integer;
  i : byte;
begin
  DayOfWeekFromDateStr := '';
  
  StrToDMY(dStr, DD, MM, YY);

  i := (
          (
            (DMYtoDate(DD, MM, YY) + 6) mod 7
          ) + 1
       );

  if (DF = ShortDay) then
    DayOfWeekFromDateStr := ShortDayNames[i]
  else
    DayOfWeekFromDateStr := LongDayNames[i];
end;

end.


